--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]

-- Bars
local _G = _G
local string_find = string.find
local string_sub = string.sub 
local math_floor = math.floor
local math_min = math.min
local GetTime = GetTime
local rBorder = { "h","h","h","h","h", }
local lBorder = { "h","h","h","h","h", }

local setBarHeight, setBarColor, flashCastBar, fadeoutCastBar
do
    function setBarHeight(bar, percent)
		--DHUD2:Debug("setBarHeight", bar, percent, DHUD2.layout.bars[bar])
        -- Hide when Bar empty 
        if math_floor(percent * 100) == 0 then
            return false
        end
        local texture = _G["DHUD2_"..bar.."_Texture"]
        local point, parent, relative, x, y, width, height, x0, x1, y0, y1, path, tex_height, tex_gap_top, tex_gap_bottom = unpack( DHUD2.layout.bars[bar] )        
        local tex_gap_top_p    = tex_gap_top / tex_height
        local tex_gap_bottom_p = tex_gap_bottom / tex_height
        local h = (tex_height - tex_gap_top - tex_gap_bottom) * percent
        local top    = 1-(percent-(tex_gap_top_p))
        local bottom = 1-tex_gap_bottom_p
        top = top  - ((tex_gap_top_p+tex_gap_bottom_p)*(1-percent))
        texture:SetHeight(h)
        texture:SetTexCoord(x0, x1, top, bottom )
        texture:SetPoint(point, _G["DHUD2_"..bar], relative, x, tex_gap_bottom)
        texture:Show()
        return true
    end 
    
    function setBarColor(bar, color)
        local texture = _G["DHUD2_"..bar.."_Texture"]
        texture:SetVertexColor(color.r, color.g, color.b)
    end
    
    function addBarBorder(bar, show, enable)
        if enable then
            if string_find(bar, "p") then
                -- Is a Pet bar
                if string_find(bar, "l") then
                    if string_find(bar, "o") then
                        lBorder[4] = show and "s" or "h"
                    else
                        lBorder[5] = show and "s" or "h"
                    end
                else
                    if string_find(bar, "o") then
                        rBorder[4] = show and "s" or "h"
                    else
                        rBorder[5] = show and "s" or "h"
                    end
                end
            else
                -- Is a target bar
                if string_find(bar, "l") then
                    if string_find(bar, "o") then
                        lBorder[1] = show and "s" or "h"
                    elseif string_find(bar, "m") then
                        lBorder[2] = show and "s" or "h"
                    else
                        lBorder[3] = show and "s" or "h"
                    end
                else
                    if string_find(bar, "o") then
                        rBorder[1] = show and "s" or "h"
                    elseif string_find(bar, "m") then
                        rBorder[2] = show and "s" or "h"
                    else
                        rBorder[3] = show and "s" or "h"
                    end
                end
            end
        else
            lBorder[1] = "h"
            lBorder[2] = "h"
            lBorder[3] = "h"
            lBorder[4] = "h"
            lBorder[5] = "h"
            rBorder[1] = "h"
            rBorder[2] = "h"
            rBorder[3] = "h"
            rBorder[4] = "h"
            rBorder[5] = "h"
        end
    end

    function flashCastBar(side)
        _G["DHUD2_"..side.."cf"]:SetAlpha(0)
        _G["DHUD2_"..side.."cf"]:Show()
        _G["DHUD2_"..side.."cf"].barFlash = true
        _G["DHUD2_"..side.."cf"].fadeAlpha = 0
        _G["DHUD2_"..side.."cf"]:SetScript("OnUpdate", DHUD2.OnUpdateFlash)
    end

end

function DHUD2:updateBar(bar, maxval, curval, fracval, color, text)
    local p = DHUD2.db.profile
    --DHUD2:Print("updateBar", bar, maxval, curval, fracval)
    if maxval then
        if setBarHeight(bar, fracval) then
            _G["DHUD2_"..bar]:Show()
            setBarColor(bar, color)
            addBarBorder(bar, true, p.borders)
            self:updateBarText(bar, color, text)
        else
            addBarBorder(bar, false, p.borders)
            self:hideBarText(bar)
            _G["DHUD2_"..bar]:Hide()
        end
    else
        addBarBorder(bar, false, p.borders)
        self:hideBarText(bar)
        _G["DHUD2_"..bar]:Hide()
    end
end

function DHUD2:initBorders()
    local i
    for i = 1, 5 do
        lBorder[i] = "h"
        rBorder[i] = "h"
    end
end

function DHUD2:addBackgroundBorder()
    -- Border is in the form 321pp, where each one has a value of s (show) or h (hide). This information is saved in a list (lBorder o rBorder) and contatenated to creat the path of the texture.
    local p = DHUD2.db.profile
    if p.borders then
        local path = ""
        for i,v in ipairs(lBorder) do path = path..v end
        if path == "hhhhh" then
            DHUD2_leftFrame_Texture:SetTexture(nil)
        else
            texture = "Interface\\AddOns\\DHUD2\\textures\\"..path
            DHUD2_leftFrame_Texture:SetTexture(texture)
            DHUD2_leftFrame_Texture:SetTexCoord(0,1,0,1)
        end
        path = ""
        for i,v in ipairs(rBorder) do path = path..v end
        if path == "hhhhh" then
            DHUD2_rightFrame_Texture:SetTexture(nil)
        else
            texture = "Interface\\AddOns\\DHUD2\\textures\\"..path
            DHUD2_rightFrame_Texture:SetTexture(texture)
            DHUD2_rightFrame_Texture:SetTexCoord(1,0,0,1)
        end
    else
        DHUD2_leftFrame_Texture:SetTexture(nil)
        DHUD2_rightFrame_Texture:SetTexture(nil)
    end
end

function DHUD2:initCast(side, name, startTime, endTime, isCast, rev)
    self:Debug("initCast", side, name, startTime, endTime, isCast)
    if startTime then
        local castBar = _G["DHUD2_"..side.."c"]
        castBar.side = side
		castBar.rev = rev
        castBar.startTime = startTime / 1000
        castBar.finishTime = endTime / 1000
        castBar.delay = 0
        castBar:SetAlpha(1)
        castBar.barFadeOut = false
        if isCast then
            castBar.isCasting = true
            castBar.isChannel = false
            setBarHeight(side.."c", 0.01)
        else
            castBar.isCasting = false
            castBar.isChannel = true
            setBarHeight(side.."c", 1)
        end
        castBar:SetScript("OnUpdate", self.OnUpdateCast) 
        castBar:Show()
        _G["DHUD2_"..side.."cf"]:Hide()
        self:updateCastText(side, 0)
        self:updateDelayText(side, 0)
		self:setCastName(side, name)
        self:updateAlpha()
    end
end

function DHUD2:OnUpdateCast(elapsed)
    local p = DHUD2.db.profile
    local currentTime = GetTime()
    local startTime = this.startTime
    local finishTime = this.finishTime
    local delay = this.delay
    if this.isCasting then
        if currentTime > finishTime then
            DHUD2:stopCast(this.side, true)
        end
        local showTime = math_min(currentTime, finishTime) - startTime
        local value = (currentTime - startTime) / (finishTime - startTime)
        if this.rev then
            cast = (finishTime - startTime) - showTime
            setBarHeight(this.side.."c", 1 - value)        
        else
            cast = showTime
            setBarHeight(this.side.."c", value)
        end
        this:Show()
        local bar = this:GetName()
        _G[bar.."_Texture"]:SetVertexColor(DHUD2:colorize("casting", "cast", value))
        DHUD2:updateCastText(this.side, cast)
        DHUD2:updateDelayText(this.side, delay)
    elseif this.isChannel then
        if currentTime > finishTime then
            DHUD2:stopCast(this.side)
        end
        local dueTime = finishTime - currentTime
        local value = dueTime / (finishTime - startTime)
        setBarHeight(this.side.."c", value)
        this:Show()
        local bar = this:GetName()
        _G[this:GetName().."_Texture"]:SetVertexColor(DHUD2:colorize("casting", "cast", value))
        DHUD2:updateCastText(this.side, dueTime)
        DHUD2:updateDelayText(this.side, delay)
    elseif this.barFadeOut then
        local fadeAlpha = this:GetAlpha() - CASTING_BAR_ALPHA_STEP
        if fadeAlpha > 0 then
            this:SetAlpha(fadeAlpha)
        else
            this:SetScript("OnUpdate", nil)
            this:Hide()
            DHUD2:updateAlpha()
        end 
    end
end

function DHUD2:delayCast(side, startTime, endTime)
    local bar = _G["DHUD2_"..side.."c"]
    local oldStart = bar.startTime
    if startTime then
        bar.startTime = startTime / 1000
        bar.finishTime = endTime / 1000
        bar.delay = bar.delay + (startTime/1000 - oldStart)
    else
        bar:Hide()
    end
end

function DHUD2:stopCast(side, isCast)
    self:Debug("StopCast", side, isCast)
    local castBar = _G["DHUD2_"..side.."c"]
    if (castBar.isCasting and isCast) or (castBar.isChannel and not isCast) then
        castBar.isCasting = false
        castBar.isChannel = false
        castBar.barFadeOut = true
		castBar.rev = false
        _G["DHUD2_"..side.."c_Texture"]:SetVertexColor(self:colorize("casting", "cast", 1))
        flashCastBar(castBar.side)
        self:updateAlpha()
        return false
    else
        return true
    end
end


function DHUD2:OnUpdateFlash(elapsed)
    --DHUD2:Debug("OnUpdateFlash", this:GetName())
    if this.barFlash then
        local fadeAlpha = this:GetAlpha() + CASTING_BAR_FLASH_STEP
        if fadeAlpha < 1 then
            this:SetAlpha(fadeAlpha)
        else
            this.barFlash = false            
            this:SetScript("OnUpdate", nil)
            this:Hide()
            DHUD2:updateAlpha()
        end            
    else
        this:SetScript("OnUpdate", nil)
    end
end
