--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]

--Addon Mix
DHUD2 = LibStub("AceAddon-3.0"):NewAddon("DHUD2", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibDebugLog-1.0")

--ACE Version
local VERSION = tonumber(("$Rev: 72 $"):match("%d+"))
DHUD2.revision = "r" .. VERSION
DHUD2.versionstring = "2.0 |cffff8888r%d|r"
DHUD2.version = DHUD2.versionstring:format(VERSION)
DHUD2.date = ("$Date: 2008-11-05 01:51:57 +0000 (Wed, 05 Nov 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD2")
local DogTag = LibStub("LibDogTag-3.0")

local unpack = unpack
local pairs = pairs
local type = type
local _G = _G
local string_find = string.find
local string_format = string.format
local string_len = string.len
local string_sub = string.sub 
local string_gsub = string.gsub
local string_match = string.match
local UnitHealthMax = UnitHealthMax
local UnitHealth = UnitHealth
local UnitInParty = UnitInParty
local UnitIsEnemy = UnitIsEnemy
local UnitManaMax = UnitManaMax
local UnitMana = UnitMana
local UnitIsTapped = UnitIsTapped
local UnitIsTappedByPlayer = UnitIsTappedByPlayer
local UnitPowerType = UnitPowerType
local UnitClassification = UnitClassification
local UnitIsPlayer = UnitIsPlayer
local UnitPlayerControlled = UnitPlayerControlled
local UnitCanAttack = UnitCanAttack
local UnitIsPVP = UnitIsPVP
local UnitCanAttack = UnitCanAttack
local UnitFactionGroup = UnitFactionGroup
local UnitReaction = UnitReaction
local UnitIsPVPFreeForAll = UnitIsPVPFreeForAll
local UnitDebuff = UnitDebuff
local UnitCreatureType = UnitCreatureType
local UnitIsUnit = UnitIsUnit
local UnitClass = UnitClass
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitPlayerOrPetInParty = UnitPlayerOrPetInParty 
local UnitBuff = UnitBuff
local GetPlayerBuff = GetPlayerBuff
local GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local GetRaidTargetIndex = GetRaidTargetIndex
local GetDifficultyColor = GetDifficultyColor
local GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local GetPetHappiness = GetPetHappiness

local isSetup = false
local disabled = false
local targetBorder = ""
local petBorder = ""
local isAlive = false
local isDead = false
local playerCast = false
local inCombat = false
local isDruid = false
local isShaman = false
local hasPet = false
local regen = false
local needH
local needP
local currLayout = {
        pH = "e",
        pP = "e",
		pC = "e",
        pI = "e",
        pB = "e",
		pE = "e",
		tH = "e",
        tP = "e",
		tI = "e",
        eH = "e",
        eP = "e",
		eI = "e",
        druidMana = "e",
        tC = "e",
        tTH = "e",
        tTP = "e",
        fH = "e",
        fP = "e",
    }

DHUD2.isShaman = isShaman
DHUD2.isWarrior = false
DHUD2.isBear = false
DHUD2.isAlive = isAlive
DHUD2.isDead = isDead
DHUD2.rBorder = rBorder
DHUD2.lBorder = lBorder
DHUD2.currLayout = currLayout
    
-- Local Functions
local showFrames, myAddons, formatTime, getHealth, getPower
do
    function myAddons()
        if (myAddOnsFrame_Register) then
            local DHUD2_mya = {
                ["name"]         = "DHUD2",
                ["version"]      = versionstring,
                ["author"]       = "Arcanefoam (Horacio Hoyos) -- Drathl/Silberklinge",
                ["category"]     = MYADDONS_CATEGORY_COMBAT,
                ["email"]        = "arcanefoam@gmail.com",
                ["website"]      = "http://www.wowace.com/projects/dhud2/",
            }
            myAddOnsFrame_Register(DHUD2_mya)
        end
    end
    
    --[[
            getHealth: Get the health value for a unit. 
            arguments:
                unit: unit to query value
            returns:
                maximum, current and percent values. need: bollean idicating if the unit's health is below 100% --]]
    function getHealth(unit)
        local maxVal, curVal, curFrac, need, found
        maxVal = UnitHealthMax(unit)
        --if unit == "player" or unit == "pet" or UnitIsUnit(unit, "pet") then
        curVal = UnitHealth(unit)
        curFrac = curVal / maxVal
        if curFrac ~= 1 then 
            need = true
        end
        return maxVal, curVal, curFrac, need
    end
    
    --[[
            getPower: Get the health value for a unit. 
            arguments:
                unit: unit to query value
                powerType: type of power for that unit
            returns:
                maximum, current and percent values. nedd: bollean idicating if the unit lackcs power (has power if power = rage) --]]
    function getPower(who)
        local maxVal, curVal, curFrac, need, powerType
        powerType = UnitPowerType(who)
        ------self:Debug("getPower", who, powerType)
        if powerType then
            maxVal = UnitManaMax(who)
            curVal = UnitMana(who)
            if maxVal ~= 0 then
                curFrac = curVal / maxVal
                if powerType == 1 then
                    if curFrac ~= 0 then 
                        need = true
                    end
                else
                    if curFrac ~= 1 then 
                        need = true
                    end
                end
            else
                curFrac = 0
                need = false
            end
            return maxVal, curVal, curFrac, need
        else
            return false
        end
    end

end
DHUD2.getHealth = getHealth
DHUD2.getPower = getPower

 -- General use functions
 --[[
            updateAlpha: Set the HUD's alpha acording to the state of the player
            arguments:
            returns:
                --]]
function DHUD2:updateAlpha()
    local p = self.db.profile
    local alpha
    if isDead then
        alpha = "deadAlpha"
    elseif inCombat then
        alpha = "combatAlpha"  
    elseif hasTarget then
        alpha = "selectAlpha"
    elseif regen or needH or needP then
        alpha = "regenAlpha"
    else
        alpha = "oocAlpha"
    end
	self:addBackgroundBorder()
    DHUD2_leftFrame:SetAlpha(p[alpha])
    DHUD2_rightFrame:SetAlpha(p[alpha])
end

--[[
            rgb2hex: Transform a RGB color to a HEX color
            arguments:
                r, g, b: RGB values
            returns:
                Hex value for the RGB   --]]
function DHUD2:rgb2hex(r, g, b)
    local hexr, hexg, hexb
    hexr = string_format("%x", r * 255)
    hexr = string_len(hexr) == 1 and "0"..hexr or hexr
    hexg = string_format("%x", g * 255)
    hexg = string_len(hexg) == 1 and "0"..hexg or hexg
    hexb = string_format("%x", b * 255)
    hexb = string_len(hexb) == 1 and "0"..hexb or hexb
    return hexr..hexg..hexb
end

--[[
        colorize: Get the bar color according to the mana or power type and percent full. USed for bars and texts
        arguments:
            who: Unit's property being colorized
            propoerty: Property being colorized
            percent: Current percent value for that property
        returns:
            r, g, b and hex values for the calculated color.    --]]
function DHUD2:colorize(what, property, percent)
	local colorTable
	if what == "player" then
		colorTable = DHUD2.db.profile.pColors
	elseif what == "target" then
		colorTable = DHUD2.db.profile.tColors
	elseif what == "pet" then
		colorTable = DHUD2.db.profile.eColors
	elseif what == "targetTarget" or what == "focus" then
		colorTable = DHUD2.db.profile.oColors
	elseif what == "casting" then
		colorTable = DHUD2.db.profile.castColors
	end
	----self:Debug("colorize", what, property, percent, colorTable)
   
	local r, g, b, diff, r0, g0, b0, r1, g1, b1, r2, g2, b2
    local threshold1, threshold2  = 0.6, 0.2
    if colorTable[tostring(property)] then
        r0, g0, b0 = unpack(colorTable[tostring(property)].full)
        r1, g1, b1 = unpack(colorTable[tostring(property)].med)
        r2, g2, b2 = unpack(colorTable[tostring(property)].low)
    end
	--self:Debug("colorize", what, property, r0, g0, b0 ,r1, g1, b1,r2, g2, b2)
    if ( percent <= threshold2 ) then
        r, g, b = r2, g2, b2
    elseif ( percent <= threshold1) then
        diff = 1 - (percent - threshold2) / (threshold1 - threshold2);
        r = r1 - (r1 - r2) * diff;        
        g = g1 - (g1 - g2) * diff;        
        b = b1 - (b1 - b2) * diff;    
    elseif ( percent < 1) then        
        diff = 1 - (percent - threshold1) / (1 - threshold1);        
        r = r0 - (r0 - r1) * diff;        
        g = g0 - (g0 - g1) * diff;        
        b = b0 - (b0 - b1) * diff;    
    else
        r, g, b = r0, g0, b0
    end
	----self:Debug("colorize", r, g, b)
    if (r < 0) then r = 0; end    
    if (r > 1) then r = 1; end    
    if (g < 0) then g = 0; end    
    if (g > 1) then g = 1; end    
    if (b < 0) then b = 0; end    
    if (b > 1) then b = 1; end
    hex = self:rgb2hex(r, g, b)
    return r, g, b, hex
end

--[[
        checkElite: Check if the unit is elite
        arguments:
            unit: unit to check
        returns:
            icon to use, text to show --]]
function DHUD2:checkElite(unit)
    local el = UnitClassification(unit)
	local eliteValues = {
            ["worldboss"] = {"targetElite", "++"},
            ["elite"] = {"targetElite", "+"},
            ["rareelite"] = {"targetRare", "r+"},
            ["rare"] = {"targetRare", "r"},
            ["normal"] = {false, ""},
            }
    if el then
		if ( ( eliteValues[el][1] ~= nil ) and ( eliteValues[el][2] ~= nil ) ) then
			return eliteValues[el][1], eliteValues[el][2]
		end
	end    
    return eliteValues["normal"][1], eliteValues["normal"][2]
    
end

 --[[
        unitIsNPC: Check if the unit is a NPC
        arguments:
            unit: unit to check
        returns:
            true if the unit is a NPC --]]
function DHUD2:unitIsNPC(unit)
    return not (UnitIsPlayer(unit) or UnitPlayerControlled(unit) or UnitCanAttack("player", unit))
end
--[[
        unitIsPet: Check if the unit is a pet
        arguments:
            unit: unit to check
        returns:
            true if the unit is a pet --]]
function DHUD2:unitIsPet(unit)
    return not self:unitIsNPC(unit) and not UnitIsPlayer(unit) and UnitPlayerControlled(unit)
end   

function DHUD2:OnInitialize()
	DHUD2:ToggleDebugLog( true )
    ----self:Debug("OnInitialize")
    self:createFrames()
	--Manage the Database
	self.db = LibStub("AceDB-3.0"):New("DHUD2DB", self.defaults, "Default")
	self.db.RegisterCallback( DHUD2, "OnProfileChanged", "OnProfileChanged" )
	self.db.RegisterCallback( DHUD2, "OnProfileCopied", "OnProfileChanged" )
	self.db.RegisterCallback( DHUD2, "OnProfileReset", "OnProfileChanged" )
	
	-- Add info to the Options array for the Profile Options
	self.options.args.profile.args.options = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profile.args.options.name = L["Options profile"]
	self.options.args.profile.args.options.desc = L["Saved profiles for DHUD2"]
	self.options.args.profile.args.options.order = 2
	
	
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("DHUD2", self.options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DHUD2", "DHUD2")
    self:RegisterChatCommand("dhud2", "ChatCommand")
    

    --[[
    --FuBar
    self.hasIcon = true
    self:SetIcon("Interface\\AddOns\\DHUD2\\icon.tga")
    self.OnMenuRequest = self.options
    ---]]
    -- Player/Target Dropdown
    UIDropDownMenu_Initialize(DHUD2_playerDropDown, playerDropDown_Init, "MENU")
    UIDropDownMenu_Initialize(DHUD2_targetDropDown, targetDropDown_Init, "MENU")
    DHUD2_target_Text:RegisterForClicks('RightButtonUp')
    DHUD2_target_Text:SetScript("OnClick", function(this, mouseButton)
        if (mouseButton == "RightButton") then
            ToggleDropDownMenu(1, nil, DHUD2_targetDropDown, this, 25, 10)
        end
    end )
    
    -- MyAddons Support
    myAddons()
end

function DHUD2:ChatCommand(input)
	----DHUD2:Debug("ChatCommand2", input)
    if not input or input:trim() == "" then
        --InterfaceOptionsFrame_OpenToCategory(DHUD2.optionsFrame)
		LibStub("AceConfigDialog-3.0"):Open("DHUD2");
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(DHUD2, "dhud2", input)
    end

end

function DHUD2:OnClick()
    if AceLibrary:HasInstance("Waterfall-1.0") then
        AceLibrary("Waterfall-1.0"):Open('DHUD2')
	end
end

function DHUD2:blizzFrames(showPlayer, showTarget, showPet, showCasting)
    if showTarget then
        TargetFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
    	TargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
    	TargetFrame:RegisterEvent("PLAYER_FOCUS_CHANGED");
    	TargetFrame:RegisterEvent("UNIT_HEALTH");
    	TargetFrame:RegisterEvent("UNIT_LEVEL");
    	TargetFrame:RegisterEvent("UNIT_FACTION");
    	TargetFrame:RegisterEvent("UNIT_CLASSIFICATION_CHANGED");
    	TargetFrame:RegisterEvent("UNIT_AURA");
    	TargetFrame:RegisterEvent("PLAYER_FLAGS_CHANGED");
    	TargetFrame:RegisterEvent("PARTY_MEMBERS_CHANGED");
    	TargetFrame:RegisterEvent("RAID_TARGET_UPDATE");
        if UnitExists("target") then TargetFrame:Show() end
        ComboFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
        ComboFrame:RegisterEvent("UNIT_COMBO_POINTS")
    else
        TargetFrame:UnregisterEvent("PLAYER_ENTERING_WORLD");
    	TargetFrame:UnregisterEvent("PLAYER_TARGET_CHANGED");
    	TargetFrame:UnregisterEvent("PLAYER_FOCUS_CHANGED");
    	TargetFrame:UnregisterEvent("UNIT_HEALTH");
    	TargetFrame:UnregisterEvent("UNIT_LEVEL");
    	TargetFrame:UnregisterEvent("UNIT_FACTION");
    	TargetFrame:UnregisterEvent("UNIT_CLASSIFICATION_CHANGED");
    	TargetFrame:UnregisterEvent("UNIT_AURA");
    	TargetFrame:UnregisterEvent("PLAYER_FLAGS_CHANGED");
    	TargetFrame:UnregisterEvent("PARTY_MEMBERS_CHANGED");
    	TargetFrame:UnregisterEvent("RAID_TARGET_UPDATE");
        ComboFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
        ComboFrame:UnregisterEvent("UNIT_COMBO_POINTS")
        TargetFrame:Hide()
        ComboFrame:Hide()
    end
    
    if showPlayer then
        PlayerFrame:RegisterEvent("UNIT_LEVEL")
        PlayerFrame:RegisterEvent("UNIT_COMBAT")
        PlayerFrame:RegisterEvent("UNIT_SPELLMISS")
        PlayerFrame:RegisterEvent("UNIT_PVP_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_MAXMANA")
        PlayerFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
        PlayerFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
        PlayerFrame:RegisterEvent("PLAYER_UPDATE_RESTING")
        PlayerFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
        PlayerFrame:RegisterEvent("PARTY_LEADER_CHANGED")
        PlayerFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
        PlayerFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
        PlayerFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
        PlayerFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
        PlayerFrameHealthBar:RegisterEvent("UNIT_HEALTH")
        PlayerFrameHealthBar:RegisterEvent("UNIT_MAXHEALTH")
        PlayerFrameManaBar:RegisterEvent("UNIT_MANA")
        PlayerFrameManaBar:RegisterEvent("UNIT_RAGE")
        PlayerFrameManaBar:RegisterEvent("UNIT_FOCUS")
        PlayerFrameManaBar:RegisterEvent("UNIT_ENERGY")
        PlayerFrameManaBar:RegisterEvent("UNIT_HAPPINESS")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXMANA")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXRAGE")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXFOCUS")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXENERGY")
        PlayerFrameManaBar:RegisterEvent("UNIT_MAXHAPPINESS")
        PlayerFrameManaBar:RegisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:RegisterEvent("UNIT_NAME_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
        PlayerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:Show()
    else
        PlayerFrame:UnregisterEvent("UNIT_LEVEL")
        PlayerFrame:UnregisterEvent("UNIT_COMBAT")
        PlayerFrame:UnregisterEvent("UNIT_SPELLMISS")
        PlayerFrame:UnregisterEvent("UNIT_PVP_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_MAXMANA")
        PlayerFrame:UnregisterEvent("PLAYER_ENTER_COMBAT")
        PlayerFrame:UnregisterEvent("PLAYER_LEAVE_COMBAT")
        PlayerFrame:UnregisterEvent("PLAYER_UPDATE_RESTING")
        PlayerFrame:UnregisterEvent("PARTY_MEMBERS_CHANGED")
        PlayerFrame:UnregisterEvent("PARTY_LEADER_CHANGED")
        PlayerFrame:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED")
        PlayerFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
        PlayerFrame:UnregisterEvent("PLAYER_REGEN_DISABLED")
        PlayerFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")
        PlayerFrameHealthBar:UnregisterEvent("UNIT_HEALTH")
        PlayerFrameHealthBar:UnregisterEvent("UNIT_MAXHEALTH")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MANA")
        PlayerFrameManaBar:UnregisterEvent("UNIT_RAGE")
        PlayerFrameManaBar:UnregisterEvent("UNIT_FOCUS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_ENERGY")
        PlayerFrameManaBar:UnregisterEvent("UNIT_HAPPINESS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXMANA")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXRAGE")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXFOCUS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXENERGY")
        PlayerFrameManaBar:UnregisterEvent("UNIT_MAXHAPPINESS")
        PlayerFrameManaBar:UnregisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:UnregisterEvent("UNIT_NAME_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_PORTRAIT_UPDATE")
        PlayerFrame:UnregisterEvent("UNIT_DISPLAYPOWER")
        PlayerFrame:Hide()
    end    
    
    if showCasting then
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
        CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
    else
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")        
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
        CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
        CastingBarFrame:Hide()
    end
    
    if showPet then
        PetFrame:RegisterEvent("UNIT_PET");
    	PetFrame:RegisterEvent("UNIT_COMBAT");
    	PetFrame:RegisterEvent("UNIT_AURA");
    	PetFrame:RegisterEvent("PET_ATTACK_START");
    	PetFrame:RegisterEvent("PET_ATTACK_STOP");
    	PetFrame:RegisterEvent("UNIT_HAPPINESS");
        if ( UnitExists("pet") ) then
            PetFrame:Show()
        end
    else
        PetFrame:UnregisterEvent("UNIT_PET");
    	PetFrame:UnregisterEvent("UNIT_COMBAT");
    	PetFrame:UnregisterEvent("UNIT_AURA");
    	PetFrame:UnregisterEvent("PET_ATTACK_START");
    	PetFrame:UnregisterEvent("PET_ATTACK_STOP");
    	PetFrame:UnregisterEvent("UNIT_HAPPINESS");
        PetFrame:Hide()
    end
    
end

-- Asing each bar the their respective tracking stat accordingly to the saved profile
function asignBars()
    local p = DHUD2.db.profile
    
    -- Player Bars
	if string_match(p.player, "c") then
		if string_match(p.pLayout, "HLPR") then
	        currLayout.pI = "l"
			currLayout.pH = "li"
	        currLayout.pP = "ri"
			currLayout.tI = "l"
	        currLayout.tH = "lm"
	        currLayout.tP = "rm"
			currLayout.pC = "r"
	        currLayout.tC = "l"   
		else
			currLayout.pI = "l"
	        currLayout.pH = "ri"
	        currLayout.pP = "li"
	        currLayout.tI = "r"
			currLayout.tH = "rm"
	        currLayout.tP = "lm"
	        currLayout.pC = "l"
	        currLayout.tC = "r"
		end
	elseif string_match(p.player, "l") then
		currLayout.pI = "l"
		currLayout.pC = "l"
		currLayout.tI = "r"
        currLayout.tC = "r"
	    if string_match(p.pLayout, "HIPM") then
			currLayout.pH = "li"
			currLayout.pP = "lm"
			currLayout.tH = "ri"
			currLayout.tP = "rm"
		else
			currLayout.pH = "lm"
            currLayout.pP = "li"
            currLayout.tH = "rm"
            currLayout.tP = "ri"
        end
	else
		currLayout.pI = "r"
		currLayout.pC = "r"
		currLayout.tI = "l"
        currLayout.tC = "l"
		if p.pLayout == "HIPM" then
			currLayout.pH = "ri"
			currLayout.pP = "rm"
			currLayout.tH = "li"
			currLayout.tP = "lm"
        else
            currLayout.pH = "rm"
            currLayout.pP = "ri"
            currLayout.tH = "lm"
            currLayout.tP = "li"
        end
    end
	
	--Bars Text Tag
	if p.pBarHText then
		local tagCode = (string_len(p.pCHStyle) > 0) and p.pCHStyle or p.pHStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.pH.."_Text_Text"], _G["DHUD2_"..currLayout.pH.."_Text_Text"], tagCode, "Unit", { unit = "player" } )
	end
	if p.pBarPText then
		local tagCode = (string_len(p.pCPStyle) > 0) and p.pCPStyle or p.pPStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.pP.."_Text_Text"], _G["DHUD2_"..currLayout.pP.."_Text_Text"], tagCode, "Unit", { unit = "player" })
	end
	if p.tBarHText then
		local tagCode = (string_len(p.tCHStyle) > 0) and p.tCHStyle or p.tHStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.tH.."_Text_Text"], _G["DHUD2_"..currLayout.tH.."_Text_Text"], tagCode, "Unit", { unit = "target" } )
	end
	if p.tBarPText then
		local tagCode = (string_len(p.tCPStyle) > 0) and p.tCPStyle or p.tPStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.tP.."_Text_Text"], _G["DHUD2_"..currLayout.tP.."_Text_Text"], tagCode, "Unit", { unit = "target" })
	end
	
	--Player Buffs and Combos/Totems/Sunders/Lacerates
	if string_match(p.pBuffsSide, "r") then
		currLayout.pB = "r"
		currLayout.pE = "l"
	else
		currLayout.pB = "l"
		currLayout.pE = "r"
	end

    -- Pet bars
	if string_match(p.pet, "c") then
		if string_match(p.eLayout, "HLPR") then
			currLayout.eI = "l"
			currLayout.eH = "lpo"
			currLayout.eP = "rpo"
		else
			currLayout.eI = "r"
			currLayout.eH = "rpo"
			currLayout.eP = "lpo"
		end
	elseif string_match(p.pet, "l") then 
		currLayout.eI = "l"
		if string_match(p.eLayout, "HIPO") then
            currLayout.eH = "lpi"
            currLayout.eP = "lpo"
        else
            currLayout.eH = "lpo"
            currLayout.eP = "lpi"
		end
	else
		currLayout.eI = "r"
		if string_match(p.eLayout, "HIPO") then
			currLayout.eH = "rpi"
            currLayout.eP = "rpo"
        else
            currLayout.eH = "rpo"
            currLayout.eP = "rpi"
        end
    end
	--Bars Text Tag
	if p.eBarHText then
	    local tagCode = (string_len(p.eCHStyle) > 0) and p.eCHStyle or p.eHStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.eH.."_Text_Text"], _G["DHUD2_"..currLayout.eH.."_Text_Text"], tagCode, "Unit", { unit = "pet" } )
	end
	if p.eBarPText then
		local tagCode = (string_len(p.eCPStyle) > 0) and p.eCPStyle or p.ePStyle
		DogTag:AddFontString(_G["DHUD2_"..currLayout.eP.."_Text_Text"], _G["DHUD2_"..currLayout.eP.."_Text_Text"], tagCode, "Unit", { unit = "pet" })
	end

	-- Druid bar
    if string_match(p.pDruSide, "r") then
        currLayout.druidMana = "rpo"
    else
        currLayout.druidMana = "lpo"
    end
    
    -- Outer bars
	currLayout.tTH = string_match(p.rightBar, "TTH") and "ro" or (string_match(p.leftBar, "TTH") and "lo" or "e")
	if currLayout.tTH ~= "e" then
		if p.oBarHText then
			local tagCode = (string_len(p.oCHStyle) > 0) and p.oCHStyle or p.oHStyle
			DogTag:AddFontString(_G["DHUD2_"..currLayout.tTH.."_Text_Text"], _G["DHUD2_"..currLayout.tTH.."_Text_Text"], tagCode, "Unit", { unit = "targettarget" })
		end
	end
	currLayout.tTP = string_match(p.rightBar, "TTP") and "ro" or (string_match(p.leftBar, "TTP") and "lo" or "e")
	if currLayout.tTP ~= "e" then
		if p.oBarPText then
			local tagCode = (string_len(p.oCPStyle) > 0) and p.oCPStyle or p.oPStyle
			DogTag:AddFontString(_G["DHUD2_"..currLayout.tTP.."_Text_Text"], _G["DHUD2_"..currLayout.tTP.."_Text_Text"], tagCode, "Unit", { unit = "targettarget" })
		end
	end
    currLayout.fH = string_match(p.leftBar, "FH") and "lo" or (string_match(p.rightBar, "FH") and "ro" or "e")
	if currLayout.fH ~= "e" then
		if p.oBarHText then
			local tagCode = (string_len(p.oCHStyle) > 0) and p.oCHStyle or p.oHStyle
			DogTag:AddFontString(_G["DHUD2_"..currLayout.fH.."_Text_Text"], _G["DHUD2_"..currLayout.fH.."_Text_Text"], tagCode, "Unit", { unit = "focus" })
		end
	end
    currLayout.fP = string_match(p.leftBar, "FP") and "lo" or (string_match(p.rightBar, "FP") and "ro" or "e")
	if currLayout.fP ~= "e" then
		if p.oBarPText then
			local tagCode = (string_len(p.oCPStyle) > 0) and p.oCPStyle or p.oPStyle
			DogTag:AddFontString(_G["DHUD2_"..currLayout.fP.."_Text_Text"], _G["DHUD2_"..currLayout.fP.."_Text_Text"], tagCode, "Unit", { unit = "focus" })
		end
	end

end

FIRE_TOTEM_SLOT = 1;
EARTH_TOTEM_SLOT = 2;
WATER_TOTEM_SLOT = 3;
AIR_TOTEM_SLOT = 4;

MAX_TOTEMS = 4;

TOTEM_PRIORITIES =
{
	AIR_TOTEM_SLOT,
	WATER_TOTEM_SLOT,
	EARTH_TOTEM_SLOT,
	FIRE_TOTEM_SLOT
};
-- Thanks to bliss totem frames
function DHUD2:totemUpdate()
	local haveTotem, name, startTime, duration, icon
	local slot
	local button
	local buttonIndex = 1
	for i=1, MAX_TOTEMS do
		slot = TOTEM_PRIORITIES[i];
		haveTotem, name, startTime, duration, icon = GetTotemInfo(slot)
		if haveTotem then
			button = _G["DHUD2_"..currLayout.pE.."b"..buttonIndex]
			button.slot = slot
            button.totemTimer = 0
			self:updateTotemButton(currLayout.pE.."b"..buttonIndex, startTime, duration, icon)
			buttonIndex = buttonIndex + 1
		else
			button = _G["DHUD2_"..currLayout.pE.."b"..MAX_TOTEMS - i + buttonIndex]
			button.slot = 0
			button:Hide()
		end
        ----self:Debug("totemUpdate", buttonIndex-1, button:IsVisible())
	end
end

-- Initialize the respective bars with their values and hide the unused ones
function DHUD2:setup()
	--self:Debug("Setup")
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile

    -- Set the HUD scale
    DHUD2_mainFrame:Show()
    DHUD2_mainFrame:SetScale(p.scale)
	
	-- Borders
    self:initBorders()
    
    -- Designate de bars accordingly to the profile
    asignBars()
	self:updateBar("lpi", false)
	self:updateBar("lpo", false)
	self:updateBar("rpi", false)
	self:updateBar("rpo", false)
	
	-- Pet	
	hasPet = UnitExists("pet")
	if hasPet and p.eBars then
		happ = GetPetHappiness()
		maxVal, curVal, curFrac, need = getHealth("pet")
		color.r, color.g, color.b, color.hex = self:colorize("pet", 7,  curFrac)
		self:updateBar(currLayout.eH, maxVal, curVal, curFrac, color, p.eBarHText)
		self:updatePetIcon(currLayout.eI, curVal, happ)
		maxVal, curVal, curFrac, need = getPower("pet")
		color.r, color.g, color.b, color.hex = self:colorize("pet", UnitPowerType("pet"),  curFrac)
		self:updateBar(currLayout.eP, maxVal, curVal, curFrac, color, p.eBarPText)
	else
		self:updatePetIcon(currLayout.eI, 0, false)
		self:updateBar(currLayout.eH, false)
		self:updateBar(currLayout.eP, false)
	end
	
	-- Specific Class functionality
	local _, class = UnitClass("player")
    if string_match(class, "DRUID") and p.class then
        -- Init druid bar
        isDruid = true
		DHUD2.isDruid = isDruid
        if p.pDruid then
            self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
			if p.pDruidText then
				local tagCode = (string_len(p.pCDStyle) > 0) and p.pCDStyle or p.pDStyle
				DogTag:AddFontString(_G["DHUD2_"..currLayout.druidMana.."_Text_Text"], _G["DHUD2_"..currLayout.druidMana.."_Text_Text"], tagCode, "Unit", { unit = "player" })
			end
			if UnitPowerType("player") ~= 0 then
                _G["DHUD2_"..currLayout.druidMana]:Show()
                curVal = UnitPower("player", 0)
				maxVal = UnitPowerMax("player", 0)
				color.r, color.g, color.b, color.hex = self:colorize("player", 0, curVal/maxVal)
				self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color, p.pDruidText)
			else
				self:updateBar(currLayout.druidMana, false)
            end
        end
        if p.class then 
            self:RegisterEvent("UNIT_COMBO_POINTS")
        end
	elseif string_match(class, "SHAMAN") and p.class then
		-- Totems
	    for i = 1, 5 do
	        _G["DHUD2_"..currLayout.pE.."b"..i]:RegisterForClicks('LeftButtonUp')
	    end
        isShaman = true
        self:totemUpdate()
		self:RegisterEvent("PLAYER_TOTEM_UPDATE")
	elseif string_match(class, "ROGUE") and p.class then
		self:RegisterEvent("UNIT_COMBO_POINTS")
	elseif string_match(class, "WARRIOR") then
		DHUD2.isWarrior = true
	end
    
	-- Player
    isAlive = not UnitIsDeadOrGhost("player")
    isDead = not isAlive
    maxVal, curVal, curFrac, need = getHealth("player")
    color.r, color.g, color.b, color.hex = self:colorize("player", 7,  curFrac)
    self:updateBar(currLayout.pH, maxVal, curVal, curFrac, color, p.pBarHText)
    maxVal, curVal, curFrac, playerNeedP = getPower("player")
    color.r, color.g, color.b, color.hex = self:colorize("player", UnitPowerType("player"),  curFrac)
    self:updateBar(currLayout.pP, maxVal, curVal, curFrac, color, p.pBarPText)
	_G["DHUD2_lpvp"]:Hide()
	_G["DHUD2_rpvp"]:Hide()
    self:updatePlayerPvP(p.pvpIcon, currLayout.pI)
	-- Player Auras
	if p.pBuffs then
		DHUD2:updatePlayerAuras(p.pBuffs, p.pBuffsFilter, currLayout.pB)
	end
	_G["DHUD2_lrest"]:Hide()
    _G["DHUD2_rrest"]:Hide()
	if IsResting() and p.restIcon then
        _G["DHUD2_"..currLayout.pI.."rest"]:Show()
    else
        _G["DHUD2_"..currLayout.pI.."rest"]:Hide()
    end
	
	_G["DHUD2_lleader"]:Hide()
	_G["DHUD2_llooter"]:Hide()
	_G["DHUD2_rleader"]:Hide()
	_G["DHUD2_rlooter"]:Hide()
    if UnitPlayerOrPetInParty("player") then
        if IsPartyLeader() then
            _G["DHUD2_"..currLayout.pI.."leader"]:Show()
            local lootmethod = GetLootMethod()
            if lootmethod == "master" then
                _G["DHUD2_"..currLayout.pI.."looter"]:Show()
            else
                _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
            end
        else
            _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
            _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
        end
    else
        _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
        _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
    end
    _G["DHUD2_lcombat"]:Hide()
	_G["DHUD2_lcombat"]:Hide()
	
	-- Only if config changed	
	if isSetup then
		--self:Debug("Setup isSetup")
		-- Player Auras
		if p.pBuffs then
			DHUD2:updatePlayerAuras(p.pBuffs, p.pBuffsFilter, currLayout.pB)
		else
			self:hidePlayerBuffs()
		end
		
	    --Target frames, only if config changed
	    self:PLAYER_TARGET_CHANGED()
	end
	-- Target and TargetofTarget Tags
	if p.tName then
		local tagCode = (string_len(p.tCStyle) > 0) and p.tCStyle or p.tStyle
		DogTag:AddFontString(_G["DHUD2_target_Text_Text"], _G["DHUD2_target_Text"], tagCode, "Unit", { unit = "target" } )
	end
	if p.totName then
		local tagCode = (string_len(p.totCStyle) > 0) and p.totCStyle or p.totStyle
		DogTag:AddFontString(_G["DHUD2_targetTarget_Text_Text"], _G["DHUD2_targetTarget_Text"], tagCode, "Unit", { unit = "targettarget" } )
	end
	
	-- Focus
    if UnitExists("focus") then
        if currLayout.fH ~= "e" then
            maxVal, curVal, curFrac = getHealth("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", 7,  curFrac)
            self:updateBar(currLayout.fH, maxVal, curVal, curFrac, color, p.oBarHText)            
        end
        if currLayout.fP ~= "e" then
            maxVal, curVal, curFrac = getPower("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", UnitPowerType("focus"), curFrac)
            self:updateBar(currLayout.fP, maxVal, curVal, curFrac, color, p.oBarPText)            
        end
    end
 
    -- Cast Bars
    ----self:Debug("Init Cast Bars")
    DHUD2_lc:Hide()
    DHUD2_rc:Hide()
    DHUD2_lcf:Hide()
    DHUD2_rcf:Hide()
    if p.pCast or p.tCast then
        self:RegisterEvent("UNIT_SPELLCAST_START")
        self:RegisterEvent("UNIT_SPELLCAST_STOP")
        self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
        self:RegisterEvent("UNIT_SPELLCAST_FAILED", "OnSpellFail")
        self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "OnSpellFail")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
        self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
    end
    
    --Set fonts
    self:setFonts(currLayout)
    
    -- Manage Blizz Frames 
    self:blizzFrames(p.blizPlayer, p.blizTarget, p.blizPet, p.blizCasting)
    
    self:updateAlpha()
    isSetup = true
			
end

function targetDropDown_Init()
    local menu;
	local name;
	local id = nil;
	if ( UnitIsUnit("target", "player") ) then
		menu = "SELF";
	elseif ( UnitIsUnit("target", "pet") ) then
		menu = "PET";
	elseif ( UnitIsPlayer("target") ) then
		id = UnitInRaid("target")
		if ( id ) then
			menu = "RAID_PLAYER"
			name = GetRaidRosterInfo(id +1);
		elseif ( UnitInParty("target") ) then
			menu = "PARTY"
		else
			menu = "PLAYER"
		end
	else
		menu = "RAID_TARGET_ICON"
		name = RAID_TARGET_ICON
	end
	if ( menu ) then
		UnitPopup_ShowMenu(DHUD2_targetDropDown, menu, "target", name, id)
	end
end


function playerDropDown_Init()
    UnitPopup_ShowMenu( DHUD2_playerDropDown, "SELF", "player" )
end
    
function DHUD2:OnEnable() -- ok
    ----self:Debug("OnEnable")
	if not isSetup then
        self:setup()
    end  
    DHUD2_mainFrame:Show()
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_ALIVE")
    self:RegisterEvent("PLAYER_UNGHOST")
    self:RegisterEvent("PLAYER_DEAD")
    self:RegisterEvent("PLAYER_UPDATE_RESTING")
    self:RegisterEvent("PLAYER_FOCUS_CHANGED")
    self:RegisterEvent("UNIT_HEALTH", "onUnitHealth")
    self:RegisterEvent("UNIT_MAXHEALTH", "onUnitHealth")
    self:RegisterEvent("UNIT_MANA", "onUnitPower",nil)
    self:RegisterEvent("UNIT_MAXMANA", "onUnitPower",nil)  
    self:RegisterEvent("UNIT_FOCUS", "onUnitPower",nil)
    self:RegisterEvent("UNIT_MAXFOCUS", "onUnitPower",nil)
    self:RegisterEvent("UNIT_RAGE", "onUnitPower",nil)
    self:RegisterEvent("UNIT_MAXRAGE", "onUnitPower",nil)
    self:RegisterEvent("UNIT_ENERGY", "onUnitPower",nil)
    self:RegisterEvent("UNIT_MAXENERGY", "onUnitPower",nil)
	self:RegisterEvent("UNIT_RUNIC_POWER", "onUnitPower",nil)
	self:RegisterEvent("UNIT_MAXRUNIC_POWER", "onUnitPower",nil)
    self:RegisterEvent("UNIT_FACTION")
    self:RegisterEvent("UNIT_AURA")
    self:RegisterEvent("UNIT_TARGET")
    self:RegisterEvent("UNIT_HAPPINESS")
    self:RegisterEvent("UNIT_PET")
    self:RegisterEvent("RAID_TARGET_UPDATE")
    self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self:RegisterEvent("UNIT_DAMAGE")
end

--Reset the init variables so the addon will reload correctly when re-enabled
function DHUD2:OnDisable()
    ----self:Debug("OnDisable")
    DHUD2_mainFrame:Hide()
end


function DHUD2:UNIT_DAMAGE(who)
    ----self:Debug("UNIT_DAMAGE", who)
    if who == "player" then
        --self:updatePlayerAuras(DHUD2.db.profile.pBuffs, currLayout.pB)
        self:updateAlpha()
    end
end
----
    
-- Load de configuration menu/window libraries and do initial setup
function DHUD2:PLAYER_ENTERING_WORLD()
    ----self:Debug("PLAYER_ENTERING_WORLD")
end

-- Update Target and TargetTarget
function DHUD2:PLAYER_TARGET_CHANGED()
    ----self:Debug("PLAYER_TARGET_CHANGED")
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile
	
    if UnitExists("target") and p.tBars and not(self:unitIsNPC("target") and not p.tNPC) then
        hasTarget = true
        maxVal, curVal, curFrac = getHealth("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", 7, curFrac)
        self:updateBar(currLayout.tH, maxVal, curVal, curFrac, color, p.tBarHText)
        maxVal, curVal, curFrac, _ = getPower("target")
        color.r, color.g, color.b, color.hex = self:colorize("target", UnitPowerType("target"), curFrac)
        self:updateBar(currLayout.tP, maxVal, curVal, curFrac, color, p.tBarPText)
        -- Casting
		if p.tCast then
	        local spell, _, channel, _, startTime, endTime = UnitCastingInfo("target")
			--DHUD2:Debug("Target Change:",spell,channel)
			local isCast = spell and not (string_match(channel, "Channeling"))
	        if isCast then
				--DHUD2:Debug("Target Change Cast")
	            self:initCast(currLayout.tC, spell, startTime, endTime, true, DHUD2.db.profile.pRev)
	        else
	            spell, _, _, _, startTime, endTime = UnitChannelInfo("target")
	            if spell then
					--DHUD2:Debug("Target Change Channel")
	                self:initCast(currLayout.tC, spell, startTime, endTime, false, DHUD2.db.profile.pRev)
	            else
	                _G["DHUD2_"..currLayout.tC.."c"]:Hide()
	                _G["DHUD2_"..currLayout.tC.."cf"]:Hide()
	            end
	        end
		else
			_G["DHUD2_"..currLayout.tC.."c"]:Hide()
	        _G["DHUD2_"..currLayout.tC.."cf"]:Hide()
		end
		-- Make name clickable
        DHUD2_target_Text:EnableMouse(true)
    else
        hasTarget = false
        self:updateBar(currLayout.tH, false)
        self:updateBar(currLayout.tP, false)
        _G["DHUD2_"..currLayout.tC.."c"]:Hide()
        _G["DHUD2_"..currLayout.tC.."cf"]:Hide()
        DHUD2_target_Text:EnableMouse(false)
        if not isShaman then
            self:updateButtons(nil, 0, currLayout.pE)
        end
    end
	self:updateTargetAuras(hasTarget and p.tAuras, p.tSwap, p.class, currLayout.pE, p.isWarrior, p.isBear and p.isDruid)
	self:updateText("target", hasTarget)
	self:updateTargetElite(currLayout.pI, hasTarget and p.tEliteIcon)
    self:updateTargetIcon(hasTarget and p.tPvpIcon)
	if isDruid or isRogue then
		local points = GetComboPoints("player")
		self:updateButtons("c", points, currLayout.pE, DHUD2.db.profile.class)
    end
	
	--Target target
    if UnitExists("targettarget") and not(self:unitIsNPC("targettarget") and not p.tNPC) and not UnitIsUnit("targettarget", "player") then
        if currLayout.tTH ~= "e" then
            maxVal, curVal, curFrac = getHealth("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", 7, curFrac)
            self:updateBar(currLayout.tTH, maxVal, curVal, curFrac, color, p.oBarHText)
        end
        if currLayout.tTP ~= "e" then
            maxVal, curVal, curFrac, _ = getPower("targettarget")
            color.r, color.g, color.b, color.hex = self:colorize("targetTarget", UnitPowerType("targettarget"), curFrac)
            self:updateBar(currLayout.tTP, maxVal, curVal, curFrac, color, p.oBarPText)
        end
        self:updateText("targetTarget", p.totName)
    else
        if currLayout.tTH ~= "e" then
            self:updateBar(currLayout.tTH, false)
        end
        if currLayout.tTP ~= "e" then
            self:updateBar(currLayout.tTP, false)
        end
        self:updateText("targetTarget", false)
    end
    self:updateAlpha()
end

function DHUD2:PLAYER_REGEN_ENABLED()
    ----self:Debug("PLAYER_REGEN_ENABLED")
    regen = true
    inCombat = false
    _G["DHUD2_"..currLayout.pI.."combat"]:Hide()
    self:updateAlpha()
end

function DHUD2:PLAYER_REGEN_DISABLED()
    ----self:Debug("PLAYER_REGEN_DISABLED")
    regen = false
    inCombat = true
    if DHUD2.db.profile.combatIcon then
        _G["DHUD2_"..currLayout.pI.."combat"]:Show()
    end
    self:updateAlpha()
end  

function DHUD2:OnProfileChanged()
	DHUD2:Debug("OnProfileChanged")
	DHUD2:setup()
end

function DHUD2:AuraTimer()
	DHUD2:CancelTimer(DHUD2.auraHandle)
	DHUD2:updatePlayerAuras(DHUD2.db.profile.pBuffs, DHUD2.db.profile.pBuffsFilter, currLayout.pB)
    DHUD2:updateAlpha()
end

function DHUD2:UNIT_COMBO_POINTS(event, who)
    ----self:Debug("UNIT_COMBO_POINTS", who)
	local points = GetComboPoints("player")
	self:updateButtons("c", points, currLayout.pE, DHUD2.db.profile.class)
end

function DHUD2:PLAYER_ALIVE()
    ----self:Debug("PLAYER_ALIVE")
    if UnitIsDeadOrGhost("player") then
        self:PLAYER_DEAD()
    else
        isAlive = true
	    isDead = false
		self:onUnitHealth("player")
		self:onUnitPower("player")
	    self:updateAlpha()
    end
end

function DHUD2:PLAYER_UNGHOST()
    ----self:Debug("PLAYER_UNGHOST")
    isAlive = true
    isDead = false
	self:onUnitHealth("player")
	self:onUnitPower("player")
    self:updateAlpha()
end

function DHUD2:PLAYER_DEAD()
    ----self:Debug("PLAYER_DEAD")
    -- First event when killed
    inCombat = false
    isDead = true
    isAlive = false
    needH = false
    needP = false
    isCasting = false
	DHUD2:updateButtons(nil, 0, currLayout.pE, false)
    DHUD2:updateAlpha()
end  

function DHUD2:PLAYER_UPDATE_RESTING()
    ----self:Debug("PLAYER_UPDATE_RESTING")
    if IsResting() and DHUD2.db.profile.restIcon and isAlive then
        _G["DHUD2_"..currLayout.pI.."rest"]:Show();
    else
        _G["DHUD2_"..currLayout.pI.."rest"]:Hide();
    end
end

function DHUD2:PLAYER_FOCUS_CHANGED()
    ----self:Debug("PLAYER_FOCUS_CHANGED")
	local p = DHUD2.db.profile
    if UnitExists("focus") then
        local color = {r=0,g=0,b=0,hex=""}
        local maxVal, curVal, curFrac
        if currLayout.fH ~= "e" then
            maxVal, curVal, curFrac = getHealth("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", 7,  curFrac)
            self:updateBar(currLayout.fH, maxVal, curVal, curFrac, color, p.oBarHText) 
        end
        if currLayout.fP ~= "e" then
            maxVal, curVal, curFrac = getPower("focus")
            color.r, color.g, color.b, color.hex = self:colorize("focus", UnitPowerType("focus"), curFrac)
            self:updateBar(currLayout.fP, maxVal, curVal, curFrac, color, p.oBarPText)
        end
    else
        if currLayout.fH ~= "e" then
            self:updateBar(currLayout.fH, false)            
        end
        if currLayout.fP ~= "e" then
            self:updateBar(currLayout.fP, false)            
        end
    end
    self:updateAlpha()
end

function DHUD2:PLAYER_TOTEM_UPDATE()
    ----self:Debug("PLAYER_TOTEM_UPDATE")
    DHUD2:totemUpdate()
end

function DHUD2:onUnitHealth(event,who)
    if who then
		local maxVal, curVal, curFrac
		local p = DHUD2.db.profile
		local color = {r=0,g=0,b=0,hex=""}
		local typ = 7
		if who == "player" then
			maxVal, curVal, curFrac, needH = getHealth("player")
			color.r, color.g, color.b, color.hex = self:colorize("player", typ, curFrac)
			self:updateBar(currLayout.pH, maxVal, curVal, curFrac, color, p.pBarHText)
		elseif who == "target" and p.tBars and not (not p.tNPC and unitIsNPC("target")) then
			maxVal, curVal, curFrac = getHealth("target")
			if (UnitIsTapped("target") and (not UnitIsTappedByPlayer("target"))) then               
				  typ = 8
			end
			color.r, color.g, color.b, color.hex = self:colorize("target", typ, curFrac)
			self:updateBar(currLayout.tH, maxVal, curVal, curFrac, color, p.tBarHText)
		elseif who == "pet" and hasPet and p.eBars then
			maxVal, curVal, curFrac, needH = getHealth("pet")
			color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curFrac)
			self:updateBar(currLayout.eH, maxVal, curVal, curFrac, color, p.eBarHText)
		end
		if UnitExists("targettarget") and not UnitIsUnit("targettarget", "player") then
			if currLayout.tTH ~= "e" then
				maxVal, curVal, curFrac = getHealth("targettarget")
				if (UnitIsTapped("targettarget") and (not UnitIsTappedByPlayer("targettarget"))) then               
					  typ = 8
				end
				color.r, color.g, color.b, color.hex = self:colorize("targetTarget", typ,  curFrac)
				self:updateBar(currLayout.tTH, maxVal, curVal, curFrac, color, p.oBarHText)            
			end
		end
		if UnitExists("focus") then
			if currLayout.fH ~= "e" then
				maxVal, curVal, curFrac = getHealth("focus")
				if (UnitIsTapped("focus") and (not UnitIsTappedByPlayer("focus"))) then               
					  typ = 8
				end
				color.r, color.g, color.b, color.hex = self:colorize("focus", typ,  curFrac)
				self:updateBar(currLayout.fH, maxVal, curVal, curFrac, color, p.oBarHText)            
			end

		end
		DHUD2:updateAlpha()
	end
end

function DHUD2:onUnitPower(event, power, who)
	----DHUD2:Debug("onUnitPower", who, power)
    if who then
		local maxVal, curVal, curFrac
		local p = DHUD2.db.profile
		local color = {r=0,g=0,b=0,hex=""}
		local typ = UnitPowerType(who)
		--DHUD2:Debug("onUnitPower", who, typ)
		if who == "player" then
			maxVal, curVal, curFrac, needP = getPower("player")
			color.r, color.g, color.b, color.hex = self:colorize("player", typ, curFrac)
			self:updateBar(currLayout.pP, maxVal, curVal, curFrac, color, p.pBarPText)
			if _G["DHUD2_"..currLayout.druidMana]:IsVisible() and isDruid then
				curVal = UnitPower("player", 0)
				maxVal = UnitPowerMax("player", 0)
				color.r, color.g, color.b, color.hex = self:colorize("player", 0, curVal/maxVal)
				self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color, p.pDruidText)
			end
		elseif who == "target" and p.tBars then
			maxVal, curVal, curFrac = getPower("target")
			color.r, color.g, color.b, color.hex = self:colorize("target", typ, curFrac)
			self:updateBar(currLayout.tP, maxVal, curVal, curFrac, color, p.tBarPText)
		elseif who == "pet" and hasPet and p.eBars then
			maxVal, curVal, curFrac, needP = getPower("pet")
			color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curFrac)
			self:updateBar(currLayout.eP, maxVal, curVal, curFrac, color, p.eBarPText)
		end
		if UnitExists("targettarget") and not UnitIsUnit("targettarget", "player") then  
			if currLayout.tTP ~= "e" then
				maxVal, curVal, curFrac = getPower("targettarget")
				color.r, color.g, color.b, color.hex = self:colorize("targetTarget", typ, curFrac)
				self:updateBar(currLayout.tTP, maxVal, curVal, curFrac, color, p.oBarPText)            
			end
		end
		if UnitExists("focus") then
			if currLayout.fP ~= "e" then
				maxVal, curVal, curFrac = getPower("focus")
				color.r, color.g, color.b, color.hex = self:colorize("focus", typ, curFrac)
				self:updateBar(currLayout.fP, maxVal, curVal, curFrac, color, p.oBarPText)            
			end
		end
		DHUD2:updateAlpha()
    end
end

function DHUD2:UNIT_FACTION(event, who)
    local p = DHUD2.db.profile
    if who == "player" then
        self:updatePlayerPvP(p.pvpIcon, currLayout.pI)
        if UnitIsUnit("player", "target") then
            self:updateTargetIcon(p.tBars and p.tPvpIcon)
        end
    elseif who == "target" then
        self:updateTargetIcon(p.tBars and p.tPvpIcon)
    end
end

function DHUD2:UNIT_AURA(event, who)
	----self:Debug("UNIT_AURA", who)
	if who == "player" and DHUD2.db.profile.pBuffs then
		DHUD2:updatePlayerAuras(DHUD2.db.profile.pBuffs, DHUD2.db.profile.pBuffsFilter, currLayout.pB)
    elseif who == "target" and DHUD2.db.profile.tBars then
	    self:updateTargetAuras(DHUD2.db.profile.tAuras, DHUD2.db.profile.tSwap, DHUD2.db.profile.class, currLayout.pE, DHUD2.isWarrior , DHUD2.isDruid and DHUD2.isBear)
    end
end

function DHUD2:UNIT_TARGET(event, who)
    local p = DHUD2.db.profile
    if who == "target" then
        if UnitExists("targetTarget") and not UnitIsUnit("targettarget", "player") then
            local color = {r=0,g=0,b=0,hex=""}
            local maxVal, curVal, curFrac
            if currLayout.tTH ~= "e" then
                maxVal, curVal, curFrac = getHealth("targettarget")
                color.r, color.g, color.b, color.hex = self:colorize("targetTarget", 7,  curFrac)
                self:updateBar(currLayout.tTH, maxVal, curVal, curFrac, color, p.oBarHText)            
            end
            if currLayout.tTP ~= "e" then
                maxVal, curVal, curFrac = getPower("targettarget")
                color.r, color.g, color.b, color.hex = self:colorize("targetTarget", UnitPowerType("targetarget"),  curFrac)
                self:updateBar(currLayout.tTP, maxVal, curVal, curFrac, color, p.oBarPText)            
            end
            self:updateText("targetTarget", p.tBarsTargetName)
        else
            if currLayout.tTH ~= "e" then
                self:updateBar(currLayout.tTH, false)            
            end
            if currLayout.tTP ~= "e" then
                self:updateBar(currLayout.tTP, false)            
            end
            DHUD2_targetTarget_Text:Hide()
        end
    end
    self:updateAlpha()
end

function DHUD2:UNIT_HAPPINESS(event, who)
    if who == "pet" then
        local happ = GetPetHappiness()
        local health = UnitHealth(who)
        self:updatePetIcon(currLayout.eI, health, happ)
    end
end

function DHUD2:UNIT_PET(event, who)
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
    local p = DHUD2.db.profile
    if who == "player" then
        hasPet = UnitExists("pet")
        DHUD2_lpi:Hide()
        DHUD2_lpo:Hide()
        DHUD2_rpi:Hide()
        DHUD2_rpo:Hide()
        if hasPet and p.eBars then
            maxVal, curVal, curFrac, needH = getHealth("pet")
            color.r, color.g, color.b, color.hex = self:colorize("pet", 7, curFrac)
            self:updateBar(currLayout.eH, maxVal, curVal, curFrac, color, p.eBarHText)
            local happ = GetPetHappiness()
            self:updatePetIcon(currLayout.eI, curVal, happ)
            maxVal, curVal, curFrac, needP = getPower("pet")
            local typ = UnitPowerType(who)
            color.r, color.g, color.b, color.hex = self:colorize("pet", typ, curFrac)
            self:updateBar(currLayout.eP, maxVal, curVal, curFrac, color, p.eBarPText)
        else
            self:updatePetIcon(currLayout.eI, 0, false)
            self:updateBar(currLayout.eH, false)
            self:updateBar(currLayout.eP, false)
        end
    end
    self:updateAlpha()
end

function DHUD2:RAID_TARGET_UPDATE()
    self:updateTargetIcon(DHUD2.db.profile.tBars and DHUD2.db.profile.tPvpIcon)
end

function DHUD2:PARTY_LOOT_METHOD_CHANGED()
    local _, master = GetLootMethod()
    if master == 0 then
        _G["DHUD2_"..currLayout.pI.."looter"]:Show()
    else
        _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
    end
end

function DHUD2:PARTY_MEMBERS_CHANGED()
    if GetNumPartyMembers() > 0 then
        if IsPartyLeader() then
            _G["DHUD2_"..currLayout.pI.."leader"]:Show()
        else
            _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
        end
    else
        _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
        _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
    end
    if TargetFrame:IsVisible() and not DHUD2.db.profile.blizTarget then
        TargetFrame:Hide()
    end
    
end

function DHUD2:PLAYER_FLAGS_CHANGED()
    --- SERA?
    if GetNumPartyMembers() > 0 then
        if IsPartyLeader() then
            _G["DHUD2_"..currLayout.pI.."leader"]:Show()
        else
            _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
        end
    else
        _G["DHUD2_"..currLayout.pI.."leader"]:Hide()
        _G["DHUD2_"..currLayout.pI.."looter"]:Hide()
    end
end

function DHUD2:UNIT_SPELLCAST_START(event, who)
    ----self:Debug("UNIT_SPELLCAST_START", who)
    if who == "player" and DHUD2.db.profile.pCast then        
        local spell, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:initCast(currLayout.pC, spell, startTime, endTime, true, DHUD2.db.profile.pRev)
        playerCast = true
    elseif who == "target" and DHUD2.db.profile.tCast then        
        local spell, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:initCast(currLayout.tC, spell, startTime, endTime, true, DHUD2.db.profile.pRev)
    end
end

function DHUD2:UNIT_SPELLCAST_STOP(event, who)
    ----self:Debug("UNIT_SPELLCAST_STOP", who)
    if who == "player" and DHUD2.db.profile.pCast then
        playerCast = self:stopCast(currLayout.pC, true)
    elseif who == "target" and DHUD2.db.profile.tCast then
        self:stopCast(currLayout.tC, true)
    end
end

function DHUD2:UNIT_SPELLCAST_DELAYED(event, who)
    ----self:Debug("UNIT_SPELLCAST_DELAYED")
    if who == "player" and DHUD2.db.profile.pCast then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.pC, startTime, endTime)
    elseif who == "target" and DHUD2.db.profile.tCast then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.tC, startTime, endTime)
    end
    
end

function DHUD2:OnSpellFail(event, who)
    ----self:Debug("OnSpellFail", who)
    if who == "player"  and DHUD2.db.profile.pCast then
        playerCast = self:stopCast(currLayout.pC, true)
    elseif who == "target" and DHUD2.db.profile.tCast then
        self:stopCast(currLayout.tC, true)  
    end
end

function DHUD2:UNIT_SPELLCAST_CHANNEL_START(event, who)
    ----self:Debug("UNIT_SPELLCAST_CHANNEL_START")
    if who == "player" and DHUD2.db.profile.pCast then
        local spell, _, _, _, startTime, endTime = UnitChannelInfo(who)
        self:initCast(currLayout.pC, spell, startTime, endTime, false, DHUD2.db.profile.pRev)
        playerCast = true
    elseif who == "target" and DHUD2.db.profile.tCast then
        local spell, _, _, _, startTime, endTime = UnitChannelInfo(who)
        self:initCast(currLayout.tC, spell, startTime, endTime, false, DHUD2.db.profile.pRev)
    end
end

function DHUD2:UNIT_SPELLCAST_CHANNEL_STOP(event, who)
    ----self:Debug("UNIT_SPELLCAST_CHANNEL_STOP")
    if who == "player" and DHUD2.db.profile.pCast then
        playerCast = self:stopCast(currLayout.pC)
    elseif who == "target" and DHUD2.db.profile.tCast then
        self:stopCast(currLayout.tC)
    end
end


function DHUD2:UNIT_SPELLCAST_CHANNEL_UPDATE(event, who)
    ----self:Debug("UNIT_SPELLCAST_CHANNEL_UPDATE")
    if who == "player" and DHUD2.db.profile.pCast then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.pC, startTime, endTime)
    elseif who == "target" and DHUD2.db.profile.tCast then
        local _, _, _, _, startTime, endTime = UnitCastingInfo(who)
        self:delayCast(currLayout.tC, startTime, endTime)
    end
end

function DHUD2:UPDATE_SHAPESHIFT_FORM(event)
    --self:Debug("UPDATE_SHAPESHIFT_FORM")
    local p = DHUD2.db.profile
    local maxVal, curVal, curFrac
    local color = {r=0,g=0,b=0,hex=""}
	if hasPet then 
		return
	elseif isDruid and p.pDruid then
		if UnitPowerType("player") == 0 then
			self:updateBar(currLayout.druidMana, false)
		else
			DHUD2.isBear = (UnitPowerType("player") == 1)
			--DHUD2:Debug("SHAPESHIFT form")
			_G["DHUD2_"..currLayout.druidMana]:Show()
			curVal = UnitPower("player", 0)
			maxVal = UnitPowerMax("player", 0)
			color.r, color.g, color.b, color.hex = self:colorize("player", 0, curVal/maxVal)
			self:updateBar(currLayout.druidMana, maxVal, curVal, curVal/maxVal, color, p.pDruidText)
		end
	end
	maxVal, curVal, curFrac, need = getPower("player")
	color.r, color.g, color.b, color.hex = self:colorize("player", UnitPowerType("player"), curFrac)
	self:updateBar(currLayout.pP, maxVal, curVal, curFrac, color, p.pBarPText)
	self:updateAlpha()
end

--[[
function DHUD2:OnTooltipUpdate()
   Tablet:SetHint(L["Open DHUD2's configuration window"])
end
]]--
