--[[
Drathals HUD2  - DHUD2
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2008 by Horacio Hoyos

This file is part of DHUD2.

    DHUD2 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD2.  If not, see <http://www.gnu.org/licenses/>.
]]

-- Texts
local _G = _G
local string_sub = string.sub
local string_gsub = string.gsub
local string_len = string.len
local string_format = string.format
local math_floor = math.floor
local styles = {
    names = {
        ["Full"] = "<level><elite> <name> [ <class><pet><npc> ]",
        ["Simple"] = "<level><elite> <name>",
        ["Small"] = "<name> [ <class><pet><npc> ]",
        ["Minimal"] = "<level> <name>",
    },
}
local classColors = {
    ["HUNTER"] = "ABD473", 
    ["WARLOCK"] = "9482C9",
    ["PRIEST"] = "FFFFFF",
    ["PALADIN"] = "F58CBA",
    ["MAGE"] = "69CCF0",
    ["ROGUE"] = "FFF569",
    ["DRUID"] = "FF7D0A",
    ["SHAMAN"] = "00DBBA",
    ["WARRIOR"] = "C79C6E",
}
local defaultfont = "Fonts/FRIZQT__.TTF"
local defaultfont_num = "Interface\\AddOns\\DHUD\\textures\\Number.TTF"


local getReactionColor
do
    local reacColors = {
        "ff0000",
        "ffff00",
        "55ff55",
        "6d6dff",
        "008800",
        "cccccc",
        }
    function getReactionColor(unit)
    	local color
        if (UnitIsTapped(unit) and (not UnitIsTappedByPlayer(unit))) then
    		color = 6
    	elseif (UnitIsPlayer(unit)) then
    		if (UnitIsPVP(unit)) then
                color = UnitCanAttack("player", unit) and 1 or 5
    		else
    			if (UnitCanAttack("player", unit) or UnitFactionGroup(unit) ~= UnitFactionGroup("player")) then
    				color = 2
    			else
    				color = 4
    			end
    		end
    	else
    		local reaction = UnitReaction(unit, "player");
    		if (reaction) then
    			if (reaction < 4) then
    				color = 1
    			elseif (reaction == 4) then
    				color = 2
    			else
    				color = 3
    			end
    		end
    	end
    	return reacColors[color];
    end
    
end

function DHUD2:updateBarText(bar, color, show)
	if show then
		_G["DHUD2_"..bar.."_Text_Text"]:SetTextColor(color.r, color.g, color.b)
	    _G["DHUD2_"..bar.."_Text"]:Show()
	else
		_G["DHUD2_"..bar.."_Text"]:Hide()
	end
end

function DHUD2:hideBarText(bar)
    _G["DHUD2_"..bar.."_Text"]:Hide()
end



--Update Target and TargetTarget text
function DHUD2:updateText(unit, show)
    if show then
        _G["DHUD2_"..unit.."_Text"]:Show()
    else
        _G["DHUD2_"..unit.."_Text"]:Hide()
    end
end

function DHUD2:updateCastText(side, value)
    --DHUD2:Debug("updateCastText")
    if value > 0 then 
        local font = _G["DHUD2_"..side.."c_Text_Text"]
        local castText = string_format("%.1f", value)
        font:SetText(castText)
        --font:SetWidth(2000)
        --local w = font:GetStringWidth() * 1.1
        --font:SetWidth(w)
        --_G["DHUD2_"..side.."c_Text"]:SetWidth(w)
        _G["DHUD2_"..side.."c_Text"]:Show()
    else
        _G["DHUD2_"..side.."c_Text"]:Hide()
    end
end

function DHUD2:updateDelayText(side, value)
    if value > 0 then
        local font = _G["DHUD2_"..side.."cd_Text_Text"]
        local delayText = string_format("%.1f", value)
        font:SetTextColor(1,0,0)
        font:SetText(delayText)
        --font:SetWidth(2000)
        --w = font:GetStringWidth() * 1.1
        --font:SetWidth(w)
        --_G["DHUD2_"..side.."cd_Text"]:SetWidth(w)
        _G["DHUD2_"..side.."cd_Text"]:Show()
    else        
        _G["DHUD2_"..side.."cd_Text"]:Hide()
    end
end

function DHUD2:setCastName(side, name)
	local font = _G["DHUD2_"..side.."cn_Text_Text"]
	font:SetTextColor(1,0,0)
    font:SetText(name)
	--font:SetWidth(2000)
	--w = font:GetStringWidth() * 1.1
	--font:SetWidth(w)
	--_G["DHUD2_"..side.."cn_Text"]:SetWidth(w)
    _G["DHUD2_"..side.."cn_Text"]:Show()
end

function DHUD2:setFonts(layout)
        local p = self.db.profile
        local font
        local w
		local outline
		outline = p.pBuffsOut and "OUTLINE" or ""
        for i = 1, 16 do
            font = _G["DHUD2_"..layout.pB.."buff"..i.."_Text"]
            font:SetFont(defaultfont, p.pBuffsSize * p.scale, outline)
            w = font:GetStringWidth() * 1.1
            font:SetWidth(w)
        end
		
		--Targer Text
		outline = p.tNOut and "OUTLINE" or ""
        DHUD2_target_Text_Text:SetFont(defaultfont, p.tNSize * p.scale, outline)
		
        --Targer Target Text
        outline = p.totNOut and "OUTLINE" or ""
		DHUD2_targetTarget_Text_Text:SetFont(defaultfont, p.totNSize * p.scale, outline)
		
		
        -- Cast Bar texts
		outline = p.pCastOut and "OUTLINE" or ""
		_G["DHUD2_"..layout.pC.."cn_Text_Text"]:SetFont(defaultfont, p.pCastSize * p.scale, outline)
        _G["DHUD2_"..layout.pC.."cn_Text_Text"]:SetWidth(200)
        _G["DHUD2_"..layout.pC.."c_Text_Text"]:SetFont(defaultfont, p.pCastSize * p.scale, outline)
        _G["DHUD2_"..layout.pC.."c_Text_Text"]:SetWidth(30)
        _G["DHUD2_"..layout.pC.."cd_Text_Text"]:SetFont(defaultfont, p.pCastSize * p.scale, outline)
        _G["DHUD2_"..layout.pC.."cd_Text_Text"]:SetWidth(30)
		
		outline = p.tCastOut and "OUTLINE" or ""
        _G["DHUD2_"..layout.tC.."cn_Text_Text"]:SetFont(defaultfont, p.tCastSize * p.scale, outline)
        _G["DHUD2_"..layout.tC.."cn_Text_Text"]:SetWidth(200)
        _G["DHUD2_"..layout.tC.."c_Text_Text"]:SetFont(defaultfont, p.tCastSize * p.scale, outline)
        _G["DHUD2_"..layout.tC.."c_Text_Text"]:SetWidth(30)
        _G["DHUD2_"..layout.tC.."cd_Text_Text"]:SetFont(defaultfont, p.tCastSize * p.scale, outline)
        _G["DHUD2_"..layout.tC.."cd_Text_Text"]:SetWidth(30)
        
        -- Player Bar's Text
		outline = p.pBarTextOut and "OUTLINE" or ""
        font = _G["DHUD2_"..layout.pH.."_Text_Text"]
		font:SetFont(defaultfont, p.pBarTextSize * p.scale, outline)
        font:SetWidth(200)
        font = _G["DHUD2_"..layout.pP.."_Text_Text"]
        font:SetFont(defaultfont, p.pBarTextSize * p.scale, outline)
        font:SetWidth(200)
        
		-- Target Bar's Text
		outline = p.tBarTextOut and "OUTLINE" or ""
        font = _G["DHUD2_"..layout.tH.."_Text_Text"]
        font:SetFont(defaultfont, p.tBarTextSize * p.scale, outline)
        font:SetWidth(200)
        font = _G["DHUD2_"..layout.tP.."_Text_Text"]
        font:SetFont(defaultfont, p.tBarTextSize * p.scale, outline)
        font:SetWidth(200)
        
		-- Pet Bar's Text
		outline = p.eBarTextOut and "OUTLINE" or ""
        font = _G["DHUD2_"..layout.eH.."_Text_Text"]
        font:SetFont(defaultfont, p.eBarTextSize * p.scale, outline)
        font:SetWidth(200)
        font = _G["DHUD2_"..layout.eP.."_Text_Text"]
        font:SetFont(defaultfont, p.eBarTextSize * p.scale, outline)
        font:SetWidth(200)
        
		
		outline = p.oBarTextOut and "OUTLINE" or ""
        if layout.tTH ~= "e" then
			font = _G["DHUD2_"..layout.tTH.."_Text_Text"]
            font:SetFont(defaultfont, p.oBarTextSize * p.scale, outline)
            font:SetWidth(200)
        end
        
        if layout.tTP ~= "e" then
            font = _G["DHUD2_"..layout.tTP.."_Text_Text"]
            font:SetFont(defaultfont, p.oBarTextSize * p.scale, outline)
            font:SetWidth(200)
        end
        
        if layout.fH ~= "e" then
            font = _G["DHUD2_"..layout.fH.."_Text_Text"]
            font:SetFont(defaultfont, p.oBarTextSize * p.scale, outline)
            font:SetWidth(200)
        end
        
        if layout.fP ~= "e" then
            font = _G["DHUD2_"..layout.fP.."_Text_Text"]
            font:SetFont(defaultfont, p.oBarTextSize * p.scale, outline)
            font:SetWidth(200)
        end
end
